#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "StatusMapCanvas.h"
#include "CIMOMHandle.h"
#include "NagiosHostDialog.h"

#include <gtkmm.h>
#include <libgnomecanvasmm/rect.h>
#include <libgnomecanvasmm/ellipse.h>
#include <libgnomecanvasmm/text.h>

#include <openwbem/OW_CIMException.hpp>
#include <openwbem/OW_CIMProperty.hpp>
#include <openwbem/OW_CIMValue.hpp>

#include <assert.h>

using namespace Glib;
using namespace Gtk;
using namespace OpenWBEM;
using namespace Gnome::Canvas;
using namespace std;

enum {
	DRAG_TARGET_TEXT_PLAIN_0,
	DRAG_TARGET_TEXT_PLAIN_1,
	DRAG_TARGET_CIMOBJECTPATH
};

StatusMapCanvas::StatusMapCanvas(const CIMInstanceHandle &handle)
{
	this->handle = handle;
	draw_map();

	// set up drag source
	vector<TargetEntry> targets;
	targets.push_back(TargetEntry("text/plain",
			0, DRAG_TARGET_TEXT_PLAIN_0));
	targets.push_back(TargetEntry("STRING",
			0, DRAG_TARGET_TEXT_PLAIN_1));
	targets.push_back(TargetEntry("text/cimobjectpath",
			0, DRAG_TARGET_CIMOBJECTPATH));
	drag_source_set(targets, Gdk::ModifierType(0),
			Gdk::ACTION_MOVE | Gdk::ACTION_COPY);

	// destination only accepts cim object paths
	vector<TargetEntry> dest_targets;
	dest_targets.push_back(TargetEntry("text/cimobjectpath",
			0, DRAG_TARGET_CIMOBJECTPATH));
	drag_dest_set(dest_targets, DEST_DEFAULT_ALL,
			Gdk::ACTION_MOVE | Gdk::ACTION_COPY);
}

void
StatusMapCanvas::draw_map()
{
	// determine size of map
	Int32 width = handle->getProperty("Width").getValue().toUInt32();
	Int32 height = handle->getProperty("Height").getValue().toUInt32();
	set_scroll_region(0.0, 0.0, (double) width, (double) height);

	// draw canvas background
	assert(root() != NULL);
	Rect *rect = new Rect(*root(), 0.0, 0.0,
			(double) width, (double) height);
	rect->property_fill_color().set_value("white");
	
	// draw hosts
	vector<CIMInstanceHandle> containments = handle->references(
		"Config4GNU_NagiosStatusMapContainment");
	for (unsigned int i = 0; i < containments.size(); i++)
	{
		drawContainment(containments[i]);
	}
}

void
StatusMapCanvas::add_host(CIMInstanceHandle host_handle,
		double xpos, double ypos)
{
	// create instance data
	CIMInstanceHandle hContainment = CIMInstanceProxy::new_instance(
			handle->getNameSpace(),
			"Config4GNU_NagiosStatusMapContainment");
	hContainment->setRefProperty("GroupComponent", handle);
	hContainment->setRefProperty("PartComponent", host_handle);
	hContainment->setProperty("XPos", CIMValue((int) xpos));
	hContainment->setProperty("YPos", CIMValue((int) ypos));
	hContainment->commitChanges();

	drawContainment(hContainment);
}

void
StatusMapCanvas::drawContainment(CIMInstanceHandle hContainment)
{
	double xpos = (double) hContainment->getProperty("XPos").getValue().toInt32();
	double ypos = (double) hContainment->getProperty("YPos").getValue().toInt32();
	CIMInstanceHandle host_handle = hContainment->getRefProperty("PartComponent");

	// determine name and status of host
	String hostname = host_handle->getProperty("ElementName")
			.getValue().toString();
	String status = host_handle->getProperty("Status")
			.getValue().toString();

	// create a group for this node
	NodeInfo info;
	info.hContainment = hContainment;
	info.grp = new Group(*root(), xpos, ypos);
	nodes.push_back(info);

	// draw circle representing status
	Ellipse *ellipse = new Ellipse(*info.grp, -12.0, -12.0,
			12.0, 12.0);
	ellipse->property_fill_color().set_value(
			status == "UP" ? "green" :
			status == "DOWN" ? "red" : "gray");
	ellipse->signal_event().connect(SigC::bind<CIMInstanceHandle>(
			SigC::slot(*this, &StatusMapCanvas::on_host_event),
			host_handle));

	// draw name of host
	Text *text = new Text(*info.grp, 0.0, 12.0,
			hostname.c_str());
}

bool
StatusMapCanvas::on_host_event(GdkEvent *event, CIMInstanceHandle host_handle)
{
	assert(event != NULL);

	if (event->type == GDK_BUTTON_PRESS)
	{
		GdkEventButton *btnevent = (GdkEventButton *) event;
		if (btnevent->button == 1)
		{
			// initiate drag
			drag_host = host_handle;
			drag_x = (int) btnevent->x;
			drag_y = (int) btnevent->y;
			g_message("initate drag %d, %d", drag_x, drag_y);

			return true;
		}
		else if (btnevent->button == 3)
		{
			// display the host dialog
			NagiosHostDialog *dlg = new NagiosHostDialog(host_handle);
			dlg->set_transient_for(*(Window *) get_toplevel());
			dlg->show();

			return true; //event has been handled
		}
	}
	else if (event->type == GDK_BUTTON_RELEASE)
	{
		if (drag_host)
		{
			g_message("button released");
			drag_host = NULL;
		}
	}
	else if (event->type == GDK_MOTION_NOTIFY)
	{
		GdkEventMotion *motionevent = (GdkEventMotion *) event;

		if (drag_host)
		{
			if (drag_check_threshold(drag_x, drag_y,
					(int) motionevent->x, (int) motionevent->y))
			{
				// begin the drag
				GtkTargetEntry targets[3];
				targets[0].target = "text/plain";
				targets[0].flags = 0;
				targets[0].info = DRAG_TARGET_TEXT_PLAIN_0;
				targets[1].target = "STRING";
				targets[1].flags = 0;
				targets[1].info = DRAG_TARGET_TEXT_PLAIN_1;
				targets[2].target = "text/cimobjectpath";
				targets[2].flags = 0;
				targets[2].info = DRAG_TARGET_CIMOBJECTPATH;
				GtkTargetList *source_target_list
					= gtk_target_list_new(targets, 3);
				drag_begin(source_target_list,
					Gdk::ACTION_MOVE | Gdk::ACTION_COPY,
					1, event);
			}
		}
		return true;
	}
	return false;
}

void
StatusMapCanvas::on_drag_data_get(const RefPtr<Gdk::DragContext> &context,
		GtkSelectionData *selection_data,
		guint info, guint time)
{
	g_message("Reached on_drag_data_get");

	String text = drag_host ?
		drag_host->getObjectPath().toString() : "";
	gtk_selection_data_set(selection_data,
			GDK_SELECTION_TYPE_STRING,
			8, // 8 bits per character
			(const guchar *) text.c_str(), text.length());
}

void
StatusMapCanvas::on_drag_data_received(const RefPtr<Gdk::DragContext> &context,
		int x, int y, GtkSelectionData *selection_data,
		guint info, guint time)
{
	g_message("Reached on_drag_data_received");

	if (selection_data == NULL)
		return;
	if (selection_data->length < 0)
		return;
	
	g_message("drop occurred at %d, %d", x, y);

	if (info == DRAG_TARGET_CIMOBJECTPATH)
	{
		// dragged a CimObjectPath
		const char *new_text = (const char *) selection_data->data;
		CIMObjectPath cop = CIMObjectPath::parse(new_text);

		context->drag_finish(true, true, time);

		// add the new host
		CIMInstanceHandle new_host_handle = CIMInstanceProxy::create(cop);
		add_host(new_host_handle, x, y);
	}
	else
	{
		// drag failed
		context->drag_finish(false, false, time);
	}
}

void
StatusMapCanvas::on_drag_data_delete(const RefPtr<Gdk::DragContext> &context)
{
	// find the containment object that points to the given host
	for (vector<NodeInfo>::iterator i = nodes.begin(); i != nodes.end(); i++)
	{
		NodeInfo info = *i;
		CIMInstanceHandle host = info.hContainment->getRefProperty(
				"PartComponent");
		if (host->equals(drag_host))
		{
			info.hContainment->destroy();
			delete info.grp;
			nodes.erase(i);
			g_message("Deleted");
			return;
		}
	}
}

void
StatusMapCanvas::on_drag_end(const RefPtr<Gdk::DragContext> &context)
{
	drag_host = NULL;
	g_message("Drag Ended");
}
