#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

// Samba plugin headers
#include "SambaPlugin.h"
#include "SambaGlobalSettingsDialog.h"
#include "SambaSharePropertiesDialog.h"
#include "SambaNewShareDialog.h"

// CIMBrowser headers
#include "ClassHandlerRegistry.h"
#include "ModifyInstanceDialog.h"
#include "InstancePropertyListForm.h"
#include "CIMOMHandle.h"

// Gtkmm headers
#include <gtkmm/messagedialog.h>

// OpenWBEM headers
#include <openwbem/OW_CIMException.hpp>
#include <openwbem/OW_CIMValue.hpp>

// System headers
#include <assert.h>

using namespace OpenWBEM;
using namespace Gtk;

/**
 * A scope item representing a CIM instance of Config4GNU_SambaService.
 * @ingroup SambaPlugin
 * This scope item adds menu items for configuring Samba and provides
 * implementations for those menu items.
 */
class SambaServiceScopeItem
	: public DefaultInstanceScopeItem
{
public:
	/**
	 * Creates a new scope item representing a Samba service.
	 */
	SambaServiceScopeItem(const CIMObjectPath &cop, MainWindow *main_window)
		: DefaultInstanceScopeItem(cop, main_window)
	{
	}

	/**
	 * Adds action menu items for configuring Samba.
	 */
	virtual void init_actions_menu(Menu &action_menu);

private:
	void do_global_settings();
	void do_default_share_settings();
	void do_new_share();
	void do_delete_share();

	void refresh()
	{
		activate();
	}
};

class SambaServiceHandler : public ClassHandler
{
public:
	virtual void do_properties(const CIMObjectPath &cop, Window *parent_window)
	{
	}

	virtual void activate(const CIMObjectPath &cop, MainWindow *main_window)
	{
		// display shares in result pane
		main_window->get_result_list()->remove_all_columns();
		main_window->get_result_list()->add_text_column("Name", "Share Name");
		main_window->get_result_list()->add_text_column("Path", "Shared Path");
		main_window->get_result_list()->add_text_column("Comment", "Comment");
		main_window->get_result_list()->associators(cop,
				"Config4GNU_SambaHostedShare");
	}
	
	virtual ScopeItemRef create_scope_item(const CIMObjectPath &cop,
		MainWindow *main_window)
	{
		return ScopeItemRef(new SambaServiceScopeItem(cop, main_window));
	}
};

class SambaShareHandler : public ClassHandler
{
	virtual void do_properties(const CIMObjectPath &cop, Window *parent_window)
	{
		// grab associated configuration object and present it
		CIMObjectPathEnumeration result =
				CIMOMHandle::associatorNamesE(cop,
					"Config4GNU_SambaShareShareSettings");
		if (result.hasMoreElements())
		{
			CIMObjectPath mcop = result.nextElement();
			SambaSharePropertiesDialog *dlg = new SambaSharePropertiesDialog(mcop);
			dlg->set_transient_for(*parent_window);
			dlg->show();
		}
		else
		{
			MessageDialog dialog(*parent_window,
					"No associated configuration.", MESSAGE_ERROR);
			dialog.run();
		}
	}
};

extern "C" {
void
init_plugin()
{		
	ClassHandlerRegistry::add_handler("Config4GNU_SambaService",
		new SambaServiceHandler);
	ClassHandlerRegistry::add_handler("Config4GNU_SambaShare",
		new SambaShareHandler);
}
} // end extern "C" block

void
SambaServiceScopeItem::init_actions_menu(Menu &action_menu)
{
	Menu::MenuList &menulist = action_menu.items();
	menulist.push_back(Menu_Helpers::MenuElem("_Global Settings...",
		SigC::slot(*this, &SambaServiceScopeItem::do_global_settings)));
	menulist.push_back(Menu_Helpers::MenuElem("_Default Share Settings...",
		SigC::slot(*this, &SambaServiceScopeItem::do_default_share_settings)));
	menulist.push_back(Menu_Helpers::MenuElem("_New Share...",
		SigC::slot(*this, &SambaServiceScopeItem::do_new_share)));
	menulist.push_back(Menu_Helpers::MenuElem("De_lete Share",
		SigC::slot(*this, &SambaServiceScopeItem::do_delete_share)));
	menulist.push_back(Menu_Helpers::SeparatorElem());
	menulist.push_back(Menu_Helpers::MenuElem("_Properties...",
		SigC::slot(*this, &DefaultInstanceScopeItem::on_menu_properties)));
}

void
SambaServiceScopeItem::do_global_settings()
{
	// grab associated configuration object and present it
	CIMObjectPathEnumeration result =
			CIMOMHandle::associatorNamesE(cop,
					"Config4GNU_SambaServiceGlobalSettings");
	if (result.hasMoreElements())
	{
		CIMObjectPath mcop = result.nextElement();
		SambaGlobalSettingsDialog *dlg = new SambaGlobalSettingsDialog(mcop);
		dlg->set_transient_for(*main_window);
		dlg->show();
	}
	else
	{
		MessageDialog dialog(*main_window,
				"No associated settings.", MESSAGE_ERROR);
		dialog.run();
	}
}

void
SambaServiceScopeItem::do_default_share_settings()
{
	// grab associated configuration object and present it
	CIMObjectPathEnumeration result =
			CIMOMHandle::associatorNamesE(cop,
					"Config4GNU_SambaServiceDefaultSettings");
	if (result.hasMoreElements())
	{
		CIMObjectPath mcop = result.nextElement();
		SambaSharePropertiesDialog *dlg = new SambaSharePropertiesDialog(mcop);
		dlg->set_transient_for(*main_window);
		dlg->show();
	}
	else
	{
		MessageDialog dialog(*main_window,
				"No associated settings.", MESSAGE_ERROR);
		dialog.run();
	}
}

void
SambaServiceScopeItem::do_new_share()
{
	SambaNewShareDialog dialog(cop);
	dialog.set_transient_for(*main_window);
	int response = dialog.run();
	if (response == RESPONSE_OK)
	{
		try
		{
			CIMInstance inst = dialog.createInstanceObject();
			CIMOMHandle::createInstance(inst);
		}
		catch (CIMException e)
		{
			MessageDialog errorDlg(*main_window, e.getMessage(), MESSAGE_ERROR);
			errorDlg.run();
		}
		// refresh shares list
		refresh();
	}
}

void
SambaServiceScopeItem::do_delete_share()
{
	CIMObjectPath share_cop = main_window->get_result_list()
			->get_selected_object();
	if (share_cop == NULL)
	{
		// no share selected
		MessageDialog dialog(*main_window, "No share selected.", MESSAGE_ERROR);
		dialog.run();
		return;
	}
	
	CIMValue name_value = share_cop.getKey("Name").getValue();
	assert(name_value);
	String share_name;
	name_value.get(share_name);

	MessageDialog dialog(*main_window,
			Glib::ustring("Delete ") + share_name.c_str() + "?",
			MESSAGE_QUESTION, BUTTONS_YES_NO);
	int response = dialog.run();
	
	if (response == RESPONSE_YES)
	{
		try
		{
			CIMOMHandle::deleteInstance(share_cop);
		}
		catch (CIMException e)
		{
			MessageDialog errorDlg(*main_window, e.getMessage(), MESSAGE_ERROR);
			errorDlg.run();
		}
		refresh();
	}
}
