// Copyright 2004 Jason Long

// This file is part of cimbrowser.
//
// Cimbrowser is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Cimbrowser is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Cimbrowser; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "ClassInformationDialog.h"

#include <iostream>
#include <openwbem/OW_Enumeration.hpp>
#include <openwbem/OW_CIMClass.hpp>
#include <openwbem/OW_CIMProperty.hpp>
#include <openwbem/OW_CIMQualifier.hpp>
#include <openwbem/OW_CIMValue.hpp>
#include <gtkmm/messagedialog.h>
#include <gtkmm/stock.h>

using namespace Gtk;
using namespace OpenWBEM;

ClassInformationDialog::ClassInformationDialog(CIMClass klass)
{
	Glib::ustring classNameUTF(klass.getName().c_str());
	set_title(classNameUTF + ": Class Information");
	set_default_size(300, 300);

	this->klass = klass;
	
	// add the notebook to the dialog
	get_vbox()->pack_start(notebook);

	// label for superclass
	superclass_label.set_text(Glib::ustring("Superclass: ")
			+ klass.getSuperClass().c_str());
	description_vbox.pack_start(superclass_label, PACK_SHRINK);

	// add the description text view and scrolled window to notebook
	description_scrolled.set_shadow_type(SHADOW_IN);
	description_scrolled.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC);
	description_scrolled.add(description_view);
	description_vbox.pack_start(description_scrolled);
	description_vbox.set_border_width(4);
	notebook.append_page(description_vbox, "Description");
	
	// get the class's description and put it in the text view
	CIMQualifier description_q = klass.getQualifier("Description");
	if (description_q != NULL)
	{
		String description;
		description_q.getValue().get(description);

		description_view.get_buffer()->set_text(description.c_str());
	}
	else
	{
		description_view.get_buffer()->set_text("");
		description_view.hide();
	}
	description_view.set_wrap_mode(WRAP_WORD);
	description_view.set_editable(false);
	
	// initialize property list view
	properties_store = ListStore::create(PropertyColumns);
	properties_view.set_model(properties_store);
	properties_view.append_column("Type", PropertyColumns.datatype);
	properties_view.append_column("Name", PropertyColumns.name);
	properties_view.append_column("Inherited?", PropertyColumns.propagated);
	properties_view.append_column("Key?", PropertyColumns.key);

	// add property list tree view and scrolled window to notebook
	properties_scrolled.add(properties_view);
	properties_scrolled.set_shadow_type(SHADOW_IN);
	properties_scrolled.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC);
	properties_scrolled.set_border_width(4);
	notebook.append_page(properties_scrolled, "Properties");

	// get a list of the class's properties and put it in the property list
	list_properties();

	properties_view.signal_row_activated().connect(
			SigC::slot(*this, &ClassInformationDialog::property_row_activated));

	// initialize associations list
	associations_list.set_base_class(klass.getName());

	// add associator list tree view and scrolled window to notebook
	associations_scrolled.add(associations_list);
	associations_scrolled.set_shadow_type(SHADOW_IN);
	associations_scrolled.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC);
	associations_scrolled.set_border_width(4);
	notebook.append_page(associations_scrolled, "Associations");

	// initialize subclasses list
	subclasses_list.set_base_class(klass.getName());

	// add subclasses list tree view and scrolled window to notebook
	subclasses_scrolled.add(subclasses_list);
	subclasses_scrolled.set_shadow_type(SHADOW_IN);
	subclasses_scrolled.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC);
	subclasses_scrolled.set_border_width(4);
	notebook.append_page(subclasses_scrolled, "Subclasses");

	// add buttons
	add_button(Stock::CLOSE, RESPONSE_CLOSE);

	// make sure all children widgets are visible
	show_all_children();
}

void
ClassInformationDialog::on_response(int response_id)
{
	if (response_id == RESPONSE_APPLY || response_id == RESPONSE_OK)
	{
		// apply changes
	}

	if (response_id == RESPONSE_CANCEL || response_id == RESPONSE_OK 
			|| response_id == RESPONSE_CLOSE)
		delete this;
}

void
ClassInformationDialog::list_properties()
{
	CIMPropertyArray result;
	result = klass.getAllProperties();
	
	for (CIMPropertyArray::iterator it = result.begin();
				it != result.end();
				it++)
	{
		CIMProperty prop = *it;
		TreeModel::iterator iter = properties_store->append();
		TreeModel::Row row = *iter;

		row[PropertyColumns.name] = prop.getName().c_str();
		CIMDataType type = prop.getDataType();
		if (type.isReferenceType())
		{
			row[PropertyColumns.datatype] =
				Glib::ustring(type.getRefClassName().c_str()) + " REF";
		}
		else
		{
			row[PropertyColumns.datatype] = type.toString().c_str();
		}
		row[PropertyColumns.propagated] = prop.getPropagated();
		row[PropertyColumns.key] = prop.isKey();
		row[PropertyColumns.property] = prop;
	}
}

void
ClassInformationDialog::property_row_activated(const TreeModel::Path &path,
		TreeViewColumn *column)
{
	TreeModel::iterator iter = properties_store->get_iter(path);
	TreeModel::Row row = *iter;

	CIMProperty property = row[PropertyColumns.property];

	CIMQualifier description_q = property.getQualifier("Description");
	if (description_q != NULL)
	{
		String description;
		description_q.getValue().get(description);

		MessageDialog dlg(description.c_str());
		dlg.run();
	}
}
