// Copyright 2004 Jason Long

// This file is part of cimbrowser.
//
// Cimbrowser is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Cimbrowser is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Cimbrowser; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "EditPropertyDialog.h"
#include "AddCIMObjectDialog.h"

#include <iostream>
#include <openwbem/OW_Enumeration.hpp>
#include <openwbem/OW_CIMClass.hpp>
#include <openwbem/OW_CIMProperty.hpp>
#include <openwbem/OW_CIMQualifier.hpp>
#include <openwbem/OW_CIMValue.hpp>
#include <gtkmm/messagedialog.h>
#include <gtkmm/stock.h>
#include <gtkmm/spinbutton.h>
#include <assert.h>

#define MININT8   -128
#define MAXINT8   127
#define MAXUINT8  255
#define MININT16  ((Int16) 0x8000)
#define MAXINT16  ((Int16) 0x7fff)
#define MAXUINT16 ((UInt16) 0xffff)
#define MININT32  ((Int32) 0x80000000)
#define MAXINT32  ((Int32) 0x7fffffff)
#define MAXUINT32 ((UInt32) 0xffffffff)

using namespace Gtk;
using namespace OpenWBEM;

/**
 * A dialog for editing a single scalar CIM property.
 * Despite its name, this dialog can handle non-string types,
 * such as integers, booleans, and perhaps even reals.
 * It will not do References or Arrays.
 */
class EditStringPropertyDialog : public EditPropertyDialog
{
protected:
	/// Allows user to type anything as a value for the property.
	Gtk::Entry *value_entry;

public:
	/// Create a new EditStringPropertyDialog.
	EditStringPropertyDialog(const CIMProperty &property);
	virtual String getStringValue() const;
};

class EditRefPropertyDialog : public EditPropertyDialog
{
protected:
	HBox value_hbox;
	Entry value_entry;
	Button pick_instance_btn;

	void on_pick_btn_clicked();

public:
	EditRefPropertyDialog(const CIMProperty &property);
	virtual String getStringValue() const;
};

/**
 * A dialog for editing a single CIM property where a list
 * of choices are available. Choices are drawn from the
 * "Values" qualifier in the CIM schema, and presented with
 * a "Combo" widget.
 */
class EditEnumPropertyDialog : public EditPropertyDialog
{
protected:
	/// Allows user to type a value or select from a list of choices.
	Combo value_combo;
	/** Create a EditEnumPropertyDialog but do not initialize the list
	 * of choices or set the current value. */
	EditEnumPropertyDialog(const CIMProperty &property);

public:
	/** Create an EditEnumPropertyDialog, initialize the list
	 * of choices, and set the current value. */
	EditEnumPropertyDialog(const CIMProperty &property,
			const CIMQualifier &values_q);
	virtual String getStringValue() const;
};

/**
 * A dialog for editing a single true/false CIM property.
 * This is simply a subclass of EditEnumPropertyDialog with
 * the choices fixed to "true" and "false".
 */
class EditBooleanPropertyDialog : public EditEnumPropertyDialog
{
public:
	/// Create a new EditBooleanPropertyDialog.
	EditBooleanPropertyDialog(const CIMProperty &property);
};

/**
 * A dialog for editing a numerical CIM property. The
 * dialog uses a SpinEntry (aka SpinButton) widget to allow
 * the user to type in a number or select a number by clicking
 * the up or down buttons. The bounds of the widget are set to
 * the minimum and maximum allowed by the data type of the
 * property (e.g. a "signed 8-bit integer" is limited to -128 to 127).
 * The dialog will also use the "MinValue" and "MaxValue" qualifiers
 * if they are available to set the bounds.
 */
class EditIntegerPropertyDialog : public EditPropertyDialog
{
protected:
	/// Allows user to type a number or scroll to the desired number.
	SpinButton value_spinner;

public:
	/// Create a new EditIntegerPropertyDialog.
	EditIntegerPropertyDialog(const CIMProperty &property);
	virtual String getStringValue() const;
//	virtual CIMValue getValue() const;
};

EditPropertyDialog::EditPropertyDialog(const CIMProperty &property)
	: description_label("Description:"),
	  value_label("Value:")
{
	set_title(property.getName().c_str());
	set_default_size(300, 300);

	this->property = property;
	
	// give the dialog some padding
	vbox.set_border_width(5);
	get_vbox()->pack_start(vbox);
	
	// add description label
	description_label.set_alignment(ALIGN_LEFT, ALIGN_CENTER);
	description_label.show();
	vbox.pack_start(description_label, PACK_SHRINK);

	// add the description text view and scrolled window
	description_view.set_wrap_mode(WRAP_WORD);
	description_view.set_editable(false);
	description_scrolled.set_shadow_type(SHADOW_IN);
	description_scrolled.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC);
	description_scrolled.add(description_view);
	vbox.pack_start(description_scrolled);
	
	// add value label
	value_label.set_alignment(ALIGN_LEFT, ALIGN_CENTER);
	value_label.show();
	vbox.pack_start(value_label, PACK_SHRINK);
	
	// add buttons
	add_button(Stock::CANCEL, RESPONSE_CANCEL);
	add_button(Stock::OK, RESPONSE_OK);
	set_default_response(RESPONSE_OK);

	// make sure all children widgets are visible
	show_all_children();
	update_description();
}

EditPropertyDialog::~EditPropertyDialog()
{
}

void
EditPropertyDialog::update_description()
{
	// get the property's "display name" and set the dialog title
	// to it
	CIMQualifier display_name_q = property.getQualifier("DisplayName");
	if (display_name_q && display_name_q.getValue())
	{
		String displayname;
		display_name_q.getValue().get(displayname);
		set_title(displayname.c_str());
	}
	
	// get the property's description and put it in the text view
	CIMQualifier description_q = property.getQualifier("Description");
	if (description_q && description_q.getValue())
	{
		String description;
		description_q.getValue().get(description);
		description_view.get_buffer()->set_text(description.c_str());
		description_scrolled.show();
	}
	else
	{
		description_view.get_buffer()->set_text("");
		description_label.hide();
		description_scrolled.hide();
	}
}

EditPropertyDialog *
EditPropertyDialog::create(const CIMProperty &property)
{
	CIMQualifier values_q = property.getQualifier("Values");
	if (values_q)
	{
		return new EditEnumPropertyDialog(property, values_q);
	}
	if (property.getDataType().getType() == CIMDataType::BOOLEAN)
	{
		return new EditBooleanPropertyDialog(property);
	}
	else if (property.getDataType().isNumericType())
	{
		return new EditIntegerPropertyDialog(property);
	}
	else if (property.getDataType().isReferenceType())
	{
		return new EditRefPropertyDialog(property);
	}
	else
	{
		return new EditStringPropertyDialog(property);
	}
}

CIMValue
EditPropertyDialog::getValue() const
{
	// if text is blank and the original value was Null, return 
	// Null CIMValue
	String newvalue(getStringValue());
	if (newvalue == "" && property.getValue())
	{
		return property.getValue();
	}
	
	if (property.getDataType().getType() == CIMDataType::STRING)
	{
		return CIMValue(newvalue);
	}
	if (property.getDataType().getType() == CIMDataType::BOOLEAN)
	{
		if (newvalue == "true")
			return CIMValue(true);
		if (newvalue == "false")
			return CIMValue(false);
		return CIMValue(CIMNULL);
	}
	if (property.getDataType().isNumericType())
	{
		if (newvalue == "")
		{
			return CIMValue(CIMNULL);
		}
		switch (property.getDataType().getType())
		{
			case CIMDataType::SINT8:
			{
				Int8 i = atoi(newvalue.c_str());
				return CIMValue(i);
			}
			case CIMDataType::UINT8:
			{
				UInt8 i = atoi(newvalue.c_str());
				return CIMValue(i);
			}
			case CIMDataType::SINT16:
			{
				Int16 i = atoi(newvalue.c_str());
				return CIMValue(i);
			}
			case CIMDataType::UINT16:
			{
				UInt16 i = atoi(newvalue.c_str());
				return CIMValue(i);
			}
			case CIMDataType::SINT32:
			{
				Int32 i = atoi(newvalue.c_str());
				return CIMValue(i);
			}
			case CIMDataType::UINT32:
			{
				UInt32 i = atoi(newvalue.c_str());
				return CIMValue(i);
			}
			default:
				// do nothing
				;
		}
	}
	else if (property.getDataType().isReferenceType())
	{
		if (newvalue == "")
		{
			return CIMValue(CIMNULL);
		}
		CIMObjectPath cop = CIMObjectPath::parse(newvalue);
		return CIMValue(cop);
	}
	
	g_warning("Unknown data type...");
	return CIMValue(CIMNULL);
}

EditStringPropertyDialog::EditStringPropertyDialog(const CIMProperty &property)
	: EditPropertyDialog(property)
{
	value_entry = manage(new Entry());
	value_entry->set_activates_default(true);
	value_entry->show();
	vbox.pack_start(*value_entry, PACK_SHRINK);

	CIMValue cimvalue = property.getValue();
	value_entry->set_text(cimvalue ? cimvalue.toString().c_str() : "");
}

String
EditStringPropertyDialog::getStringValue() const
{
	return String(value_entry->get_text().c_str());
}

EditRefPropertyDialog::EditRefPropertyDialog(const CIMProperty &property)
	: EditPropertyDialog(property),
	  pick_instance_btn("Build...")
{
	value_hbox.show();
	vbox.pack_start(value_hbox, PACK_SHRINK);

	CIMValue cimvalue = property.getValue();
	if (cimvalue)
	{
		CIMObjectPath cop = cimvalue.toCIMObjectPath();
		value_entry.set_text(cop.toString().c_str());
	}
	value_entry.set_activates_default(true);
	value_entry.show();
	value_hbox.pack_start(value_entry);
	
	pick_instance_btn.signal_clicked().connect(
			SigC::slot(*this, &EditRefPropertyDialog::on_pick_btn_clicked));
	pick_instance_btn.show();
	value_hbox.pack_start(pick_instance_btn, PACK_SHRINK);
}

void
EditRefPropertyDialog::on_pick_btn_clicked()
{
	AddCIMObjectDialog dialog;
	dialog.set_transient_for(*this);
	if (dialog.run() == RESPONSE_OK)
	{
		CIMObjectPath cop = dialog.get_object_path();
		value_entry.set_text(cop.toString().c_str());
	}
}

String
EditRefPropertyDialog::getStringValue() const
{
	return String(value_entry.get_text().c_str());
}

EditEnumPropertyDialog::EditEnumPropertyDialog(
		const CIMProperty &property)
	: EditPropertyDialog(property)
{
	value_combo.show();
	vbox.pack_start(value_combo, PACK_SHRINK);
}

EditEnumPropertyDialog::EditEnumPropertyDialog(
		const CIMProperty &property, const CIMQualifier &values_q)
	: EditPropertyDialog(property)
{
	value_combo.show();
	vbox.pack_start(value_combo, PACK_SHRINK);

	assert(values_q);
	assert(values_q.getValue());
	StringArray values_array = values_q.getValue().toStringArray();

	std::vector<Glib::ustring> popdown_strings;
	for (StringArray::const_iterator iter = values_array.begin();
			iter != values_array.end(); iter++)
	{
		const String &value_string = *iter;
		popdown_strings.push_back(value_string.c_str());
	}
	value_combo.set_popdown_strings(popdown_strings);
	value_combo.set_value_in_list(true, true);

	CIMValue cimvalue = property.getValue();
	value_combo.get_entry()->set_text(cimvalue ? cimvalue.toString().c_str() : "");
}

String
EditEnumPropertyDialog::getStringValue() const
{
	return String(value_combo.get_entry()->get_text().c_str());
}

EditBooleanPropertyDialog::EditBooleanPropertyDialog(
		const CIMProperty &property)
	: EditEnumPropertyDialog(property)
{
	std::vector<Glib::ustring> popdown_strings;
	popdown_strings.push_back("true");
	popdown_strings.push_back("false");
	value_combo.set_popdown_strings(popdown_strings);
	value_combo.set_value_in_list(true, true);

	CIMValue cimvalue = property.getValue();
	value_combo.get_entry()->set_text(cimvalue ? cimvalue.toString().c_str() : "");
}

EditIntegerPropertyDialog::EditIntegerPropertyDialog(
		const CIMProperty &property)
	: EditPropertyDialog(property)
{
	// initialize spinner widget
	value_spinner.set_increments(1.0, 100.0);
	value_spinner.show();
	vbox.pack_start(value_spinner, PACK_SHRINK);

	// determine acceptable range
	double minValue, maxValue;
	switch (property.getDataType().getType())
	{
	case CIMDataType::SINT8:
		minValue = double(MININT8);
		maxValue = double(MAXINT8);
		break;
	case CIMDataType::UINT8:
		minValue = 0.0;
		maxValue = double(MAXUINT8);
		break;
	case CIMDataType::SINT16:
		minValue = double(MININT16);
		maxValue = double(MAXINT16);
		break;
	case CIMDataType::UINT16:
		minValue = 0.0;
		maxValue = double(MAXUINT16);
		break;
	case CIMDataType::SINT32:
		minValue = double(MININT32);
		maxValue = double(MAXINT32);
		break;
	case CIMDataType::UINT32:
		minValue = 0.0;
		maxValue = double(MAXUINT32);
		break;
	default:
		minValue = -G_MAXDOUBLE;
		maxValue = G_MAXDOUBLE;
		break;
	}
	
	// check for MinValue and MaxValue qualifiers
	CIMQualifier minvalue_q = property.getQualifier("MinValue");
	if (minvalue_q && minvalue_q.getValue())
	{
		Int64 i;
		minvalue_q.getValue().get(i);
		minValue = double(i);
	}
	CIMQualifier maxvalue_q = property.getQualifier("MaxValue");
	if (maxvalue_q && maxvalue_q.getValue())
	{
		Int64 i;
		maxvalue_q.getValue().get(i);
		maxValue = double(i);
	}
	value_spinner.set_range(minValue, maxValue);
	
	// update spinner widget with current value of property
	CIMValue cimvalue = property.getValue();
	if (cimvalue)
	{
		switch (property.getDataType().getType())
		{
		case CIMDataType::SINT8:
			value_spinner.set_value(double(cimvalue.toInt8()));
			break;
		case CIMDataType::UINT8:
			value_spinner.set_value(double(cimvalue.toUInt8()));
			break;
		case CIMDataType::SINT16:
			value_spinner.set_value(double(cimvalue.toInt16()));
			break;
		case CIMDataType::UINT16:
			value_spinner.set_value(double(cimvalue.toUInt16()));
			break;
		case CIMDataType::SINT32:
			value_spinner.set_value(double(cimvalue.toInt32()));
			break;
		case CIMDataType::UINT32:
			value_spinner.set_value(double(cimvalue.toUInt32()));
			break;
		case CIMDataType::SINT64:
			value_spinner.set_value(double(cimvalue.toInt64()));
			break;
		case CIMDataType::UINT64:
			value_spinner.set_value(double(cimvalue.toUInt64()));
			break;
		default:
			;
		}
	}
	else
	{
		value_spinner.set_text("");
	}
}

String
EditIntegerPropertyDialog::getStringValue() const
{
	return String(value_spinner.get_text().c_str());
}
