// Copyright 2004 Jason Long

// This file is part of cimbrowser.
//
// Cimbrowser is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Cimbrowser is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Cimbrowser; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "InstancePropertyList.h"
#include "CIMOMHandle.h"
#include "EditPropertyDialog.h"

#include <gtkmm/messagedialog.h>
#include <gtkmm/stock.h>

#include <openwbem/OW_CIMValue.hpp>
#include <openwbem/OW_CIMQualifier.hpp>
#include <openwbem/OW_CIMException.hpp>
#include <openwbem/OW_CIMClass.hpp>
#include <openwbem/OW_Bool.hpp>

#include <assert.h>

#define RIGHT_BUTTON 3

using namespace Gtk;
using namespace OpenWBEM;

InstancePropertyList::InstancePropertyList()
{
	initialize();
}

InstancePropertyList::InstancePropertyList(CIMInstanceHandle instHandle)
{
	initialize();
	list_properties(instHandle);
}

InstancePropertyList::InstancePropertyList(CIMObjectPath cop)
{
	initialize();

	CIMInstanceHandle instHandle = CIMInstanceProxy::create(cop);
	list_properties(instHandle);
}

InstancePropertyList::InstancePropertyList(CIMObjectPath cop,
		Reference<CIMInstance> inst)
{
	initialize();

	CIMInstanceHandle instHandle = CIMInstanceProxy::create(cop);
	list_properties(instHandle);
}

InstancePropertyList::~InstancePropertyList()
{
}

void
InstancePropertyList::initialize()
{
	store = ListStore::create(PropertyColumns);
	set_model(store);
	append_column("Type", PropertyColumns.datatype);
	append_column("Name", PropertyColumns.name);
	append_column("Value", PropertyColumns.value);
	dirty = false;
}

void
InstancePropertyList::list_properties(CIMInstanceHandle instHandle)
{
	this->instHandle = instHandle;
	
	CIMPropertyArray result;
	result = instHandle->getProperties();
	
	for (CIMPropertyArray::iterator it = result.begin();
				it != result.end();
				it++)
	{
		CIMProperty prop = *it;
		TreeModel::iterator iter = store->append();
		TreeModel::Row row = *iter;

		setRowData(row, prop);
	}
}

void
InstancePropertyList::setRowData(TreeModel::Row &row, const CIMProperty &prop)
{
	row[PropertyColumns.name] = prop.getName().c_str();
	CIMDataType type = prop.getDataType();
	if (type.isReferenceType())
	{
		row[PropertyColumns.datatype] =
			Glib::ustring(type.getRefClassName().c_str()) + " REF";
	}
	else
	{
		row[PropertyColumns.datatype] = type.toString().c_str();
	}
	CIMValue value = prop.getValue();
	row[PropertyColumns.value] = value ?
			value.toString().c_str() : "";
	row[PropertyColumns.property] = prop;
}

void
InstancePropertyList::on_row_activated(const TreeModel::Path &path,
		TreeViewColumn *column)
{
	TreeModel::iterator iter = store->get_iter(path);
	TreeModel::Row row = *iter;

	CIMProperty property = row[PropertyColumns.property];
	EditPropertyDialog *dlg = EditPropertyDialog::create(property);
	dlg->set_transient_for(*(Dialog *) get_toplevel());

	int response = dlg->run();
	if (response == RESPONSE_OK)
	{
		CIMValue newValue = dlg->getValue();
		instHandle->setProperty(property.getName(), newValue);
		setRowData(row, instHandle->getProperty(property.getName()));
		dirty = true;
	}
	delete dlg;
}

void
InstancePropertyList::applyChanges()
{
	// call modify operation
	instHandle->commitChanges();
}

bool
InstancePropertyList::on_button_press_event(GdkEventButton *event)
{
	assert(event != NULL);
	
	// do superclass processing
	bool result = TreeView::on_button_press_event(event);
	if (event->button == RIGHT_BUTTON)
	{
		Menu *popup_menu = manage(new Menu);
		popup_menu->items().push_back(
				Menu_Helpers::MenuElem("_Edit...",
				SigC::slot(*this, &InstancePropertyList::on_menu_edit)));
		popup_menu->items().push_back(
				Menu_Helpers::StockMenuElem(Stock::CLEAR,
				SigC::slot(*this, &InstancePropertyList::on_menu_clear)));
		popup_menu->popup(event->button, event->time);
		return true;
	}
	return result;
}

void
InstancePropertyList::on_menu_edit()
{
	TreePath path;
	TreeViewColumn *column;
	get_cursor(path, column);
	if (path.gobj() == NULL)
		return;
	
	on_row_activated(path, column);
}

void
InstancePropertyList::on_menu_clear()
{
	TreePath path;
	TreeViewColumn *column;
	get_cursor(path, column);
	if (path.gobj() == NULL)
		return;
	
	TreeModel::iterator iter = store->get_iter(path);
	TreeModel::Row row = *iter;

	CIMProperty property = row[PropertyColumns.property];
	instHandle->setProperty(property.getName(), CIMValue(CIMNULL));
	setRowData(row, instHandle->getProperty(property.getName()));
}
