// Copyright 2004 Jason Long

// This file is part of cimbrowser.
//
// Cimbrowser is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Cimbrowser is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Cimbrowser; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

// CIMBrowser header files
#include "MainWindow.h"
#include "ConnectDialog.h"
#include "SelectClassDialog.h"
#include "ClassInformationDialog.h"
#include "CIMOMHandle.h"
#include "AddCIMObjectDialog.h"
#include "ClassHandlerRegistry.h"
#include "DefaultInstanceScopeItem.h"
#include "PropertyPagesDialog.h"
#include "InstancePropertyListForm.h"

// Gtk+ header files
#include <gtkmm.h>

// OpenWBEM header files
#include <openwbem/OW_CIMProtocolIFC.hpp>
#include <openwbem/OW_HTTPClient.hpp>
#include <openwbem/OW_CIMXMLCIMOMHandle.hpp>
#include <openwbem/OW_CIMException.hpp>
#include <openwbem/OW_CIMValue.hpp>

// System header files
#include <assert.h>

using namespace Gtk;
using namespace Glib;
using namespace OpenWBEM;

MainWindow::MainWindow()
{
	set_title("CIMBrowser");
	set_default_size(500, 300);

	// File menu:
	{
		Menu::MenuList &menulist = menu_console.items();
		
		menulist.push_back(Menu_Helpers::MenuElem("_Add CIM Object...",
				SigC::slot(*this, &MainWindow::on_menu_console_add_cim_object)));
		menulist.push_back(Menu_Helpers::MenuElem("Add _Container...",
				SigC::slot(*this, &MainWindow::on_menu_console_add_container)));
		menulist.push_back(Menu_Helpers::StockMenuElem(Stock::QUIT,
					SigC::slot(*this, &MainWindow::on_menu_console_quit)));
	}
	
	// Schema menu:
	{
		Menu::MenuList &menulist = menu_schema.items();
		
		menulist.push_back(Menu_Helpers::MenuElem("_Find Class...",
					SigC::slot(*this, &MainWindow::on_menu_schema_find_class)));
	}

	// Help menu:
	{
		Menu::MenuList &menulist = menu_help.items();
		
		menulist.push_back(Menu_Helpers::MenuElem("_About",
					SigC::slot(*this, &MainWindow::on_menu_help_about)));
	}
	
	menu_bar.items().push_back(Menu_Helpers::MenuElem("_Console",
			menu_console));
	menu_bar.items().push_back(Menu_Helpers::MenuElem("_Scope", menu_scope));
	menu_bar.items().push_back(Menu_Helpers::MenuElem("_Result", menu_result));
	menu_bar.items().push_back(Menu_Helpers::MenuElem("Sc_hema",
			menu_schema));
	menu_bar.items().push_back(Menu_Helpers::MenuElem("_Help", menu_help));
	
	vbox.pack_start(menu_bar, PACK_SHRINK);
	
	// prepare scope pane
	scope_list.main_window = this;
	scope_list.set_headers_visible(false);
	scope_list.append_column("Scope", ScopeColumns.name);
	scope_list.signal_row_activated().connect(
			SigC::slot(*this, &MainWindow::on_scope_row_activated));
	scope_scrolled.set_shadow_type(SHADOW_IN);
	scope_scrolled.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC);
	scope_scrolled.add(scope_list);

	// prepare instances list
	result_scrolled.set_shadow_type(SHADOW_IN);
	result_scrolled.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC);
	result_scrolled.add(result_list);
	
	// put in the HPaned widget
	paned.add1(scope_scrolled);
	paned.add2(result_scrolled);
	paned.set_position(200);
	vbox.pack_start(paned);
	
	add(vbox);
	show_all_children();
}

void
MainWindow::enum_instances(const String &className)
{
	result_list.enum_instances(className);
}

void
MainWindow::on_menu_console_add_cim_object()
{
	AddCIMObjectDialog dialog;
	dialog.set_transient_for(*this);
	if (dialog.run() == RESPONSE_OK)
	{
		CIMObjectPath cop = dialog.get_object_path();
		g_message("Adding object with CIMObjectPath of \"%s\"",
			cop.toString().c_str());

		TreeIter iter = scope_store->append();
		TreeRow row = *iter;
		
		row[ScopeColumns.name] = "New CIM Object";
		ClassHandler *handler = ClassHandlerRegistry::get_handler(
				cop.getClassName());
		assert(handler != NULL);

		row[ScopeColumns.scopeitem] = handler->create_scope_item(cop, this);

		// try getting instance's elementname property
		try
		{
			CIMInstance inst = CIMOMHandle::getInstance(cop);
			CIMProperty prop = inst.getProperty("ElementName");
			if (prop != NULL)
			{
				CIMValue value = prop.getValue();
				if (value)
				{
					row[ScopeColumns.name] = value.toString().c_str();
				}
			}
		}
		catch (...)
		{
		}
	}
}

void
MainWindow::on_menu_console_add_container()
{
	SelectClassDialog dialog;
	dialog.set_title("Add Container");
	dialog.set_transient_for(*this);
	if (dialog.run() == RESPONSE_OK)
	{
		String className = dialog.get_class_name();
		TreeIter iter = scope_store->append();
		TreeRow row = *iter;
		row[ScopeColumns.name] = className.c_str();
		ClassScopeItem *item = new ClassScopeItem(
			className, "ElementName:Name", this);
		item->expander = true;
		row[ScopeColumns.scopeitem] = ScopeItemRef(item);
	}
}

void
MainWindow::on_menu_console_quit()
{
	hide();
}

void
MainWindow::on_menu_schema_find_class()
{
	SelectClassDialog dialog;
	dialog.set_title("Find Class");
	dialog.set_transient_for(*this);
	if (dialog.run() == RESPONSE_OK)
	{
		try
		{
			CIMClass class_object = CIMOMHandle::getClass(
					dialog.get_class_name());
			ClassInformationDialog *dialog =
				new ClassInformationDialog(class_object);
			dialog->set_transient_for(*this);
			dialog->show();
		}
		catch (CIMException e)
		{
			MessageDialog dialog(*this, e.getMessage(), MESSAGE_ERROR);
			dialog.run();
		}
	}
}

void
MainWindow::on_menu_help_about()
{
}

void
MainWindow::init_scope_list()
{
	scope_store = TreeStore::create(ScopeColumns);
	scope_list.set_model(scope_store);

	TreeModel::iterator iter = scope_store->append();
	TreeModel::Row row = *iter;
	row[ScopeColumns.name] = "Computer Systems";
	ClassScopeItem *item = new ClassScopeItem(
			"CIM_ComputerSystem", "ElementName:Name", this);
	item->expander = true;
	row[ScopeColumns.scopeitem] = ScopeItemRef(item);

	item->activate();
	item->expand_row(row);
	
	scope_list.expand_row(scope_store->get_path(iter), false);
}

void
MainWindow::add_scope_item(TreeRow &row, ustring name, ScopeItemRef scopeitem)
{
	for (TreeModel::iterator iter = row.children().begin();
			iter != row.children().end(); iter++)
	{
		TreeRow cmprow = *iter;
		ustring cmpname = cmprow[ScopeColumns.name];
		if (cmpname == name)
		{
			cmprow[ScopeColumns.scopeitem] = scopeitem;
			return;
		}
	}

	// didn't find the item, so append it to the end
	TreeModel::iterator subiter = scope_store->append(row.children());
	TreeModel::Row subrow = *subiter;
	subrow[ScopeColumns.name] = name;
	subrow[ScopeColumns.scopeitem] = scopeitem;
}

void
MainWindow::on_scope_row_activated(const TreeModel::Path &path,
		TreeViewColumn *column)
{
	scope_current = scope_store->get_iter(path);
	TreeModel::Row row = *scope_current;

	ScopeItemRef scopeitem = row[ScopeColumns.scopeitem];
	if (scopeitem)
	{
		// tell scope item to display itself in the result pane
		get_result_list()->remove_all_columns();
		scopeitem->activate();
		
		// have scope item insert child items into the current row, then
		// display the children of this row
		scopeitem->expand_row(row);
		scope_list.expand_row(path, false);

		// clear current action menu and allow scope item to add items to it
		Menu::MenuList &menulist = menu_scope.items();
		menulist.erase(menulist.begin(), menulist.end());
		scopeitem->init_actions_menu(menu_scope);
	}
}

bool
MainWindow::ScopeTreeView::on_button_press_event(GdkEventButton *event)
{
	assert(event != NULL);
	assert(main_window != NULL);
	
	// do default handler
	TreeView::on_button_press_event(event);

	// check which button is pressed
	if (event->button == 3)
	{
		main_window->menu_scope.popup(event->button, event->time);
		return true;
	}
	return false;
}

MainWindow::SimpleScopeItem::SimpleScopeItem(const String &columns)
{
	this->columns = columns;
}

void
MainWindow::SimpleScopeItem::setup_columns(MainWindow *window)
{
	assert(window != NULL);

	window->result_list.remove_all_columns();
	StringArray parts = columns.tokenize(",");
	for (unsigned int i = 0; i < parts.size(); i++)
	{
		StringArray parts2 = parts[i].tokenize(":");
		window->result_list.add_text_column(parts2[0], parts2[1].c_str());
	}
}

MainWindow::ClassScopeItem::ClassScopeItem(const String &className,
		const String &columns, MainWindow *window)
		: SimpleScopeItem(columns)
{
	this->className = className;
	this->expander = false;
	this->main_window = window;
}

void
MainWindow::ClassScopeItem::init_actions_menu(Menu &action_menu)
{
	Menu::MenuList &menulist = action_menu.items();
	menulist.push_back(Menu_Helpers::MenuElem("Create Instance",
		SigC::slot(*this, &MainWindow::ClassScopeItem::on_menu_new)));
}

void
MainWindow::ClassScopeItem::on_menu_new()
{
	SelectClassDialog dialog;
	dialog.set_title("Create Instance");
	dialog.set_base_class_name(className);
	dialog.set_transient_for(*main_window);
	if (dialog.run() == RESPONSE_OK)
	{
		dialog.hide();
		String className = dialog.get_class_name();
		CIMInstanceHandle handle = CIMInstanceProxy::new_instance(
				CIMOMHandle::getDefaultNamespace(),
				className);

		PropertyPagesDialog *dlg = new PropertyPagesDialog;
		dlg->set_title("Create Instance");
		dlg->set_transient_for(*main_window);
		dlg->add_form(new InstancePropertyListForm(handle),
			"Properties");
		dlg->show();
	}
}

void
MainWindow::ClassScopeItem::activate()
{
	assert(main_window != NULL);

	setup_columns(main_window);
	main_window->enum_instances(className);
}

void
MainWindow::ClassScopeItem::expand_row(TreeModel::Row &row)
{
	if (!expander)
		return;

	try
	{
		CIMInstanceEnumeration result;
		result = CIMOMHandle::enumInstancesE(className);

		while (result.hasMoreElements())
		{
			CIMInstance inst = result.nextElement();
			CIMObjectPath cop(CIMOMHandle::getDefaultNamespace(), inst);
			CIMValue value = inst.getProperty("ElementName").getValue();
			if (value)
			{
				String str;
				value.get(str);

				ClassHandler *handler = ClassHandlerRegistry::get_handler(
							cop.getClassName());
				assert(handler != NULL);

				ScopeItemRef scopeitem = handler->create_scope_item(cop,
						main_window);
				main_window->add_scope_item(row, str.c_str(), scopeitem);
			}
		} // while has more elements
	}
	catch (CIMException e)
	{
		g_warning("%s", e.getMessage());
	}
}
